/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.oasisopen.sca.ResponseDispatch;
import org.oasisopen.sca.annotation.*;


/**
 * Java component implementation
 * 1 service with interface Service1AsyncServer
 * 0 references
 *
 * Async server implementation of the Service1 service - but using a Java interface that incorrectly does not have the 
 * @AsyncInvocation annotation applied.
 *
 */
@Service(Service1AsyncServerError.class)
public class Service1AsyncServerErrorImpl implements Service1AsyncServerError {
	
	private volatile ResponseDispatch<String> responseHandler = null;
	
	@Property(required=true)
	public volatile String serviceName = "service1";
	
	/*
	public String operation1(String input) {
		return serviceName + " operation1 invoked";
	}
	*/

	public void operation1Async(String input, ResponseDispatch<String> handler) {
		// Store the ResponseDispatch object
		responseHandler = handler;
		
		// Now kick off processing on a separate thread that will dispatch the response some time after this
		// initial method returns...
		runResponseThread();
		
		// return
		return;
	} // end method operation1Async
	
	/**
	 * Method used to run a separate thread, to invoke the ResponseDispatch
	 */
	private void runResponseThread() {
		
		int invocationCount = 2;	// # of threads to use
		long maxWaitTime = 5000;	// Max wait time for completion = 5sec
		long pauseTime = 1000;		// Pause interval to allow initiating thread to complete
		
		// Run the tests using a ThreadPoolExecutor
		ThreadPoolExecutor theExecutor = new ThreadPoolExecutor( 	invocationCount, invocationCount,
													                maxWaitTime, TimeUnit.MILLISECONDS,
													                new ArrayBlockingQueue<Runnable>( invocationCount ) );
		
		// Wait for a short time to ensure that the invoking thread has time to return
		try {
			this.wait(pauseTime);
		} catch (InterruptedException e) {
			// Nothing to do here...
		}
		
		// Perform the invocations on separate thread...
		theExecutor.execute( new separateThreadInvoker() );

	} // end method runResponseThread

	/**
	 * An inner class which acts as a runnable task for invoking APIs on threads that are not processing
	 * either a service operation or a callback operation
	 */
	private class separateThreadInvoker implements Runnable {
				
		public separateThreadInvoker() {
			super();
		} // end constructor

		public void run() {

			// Invoke the reponse dispatch object to return the response
			responseHandler.sendResponse( serviceName + " operation1 invoked asynchronously");

		} // end method run
		
	} // end class separateThreadInvoker

}
