/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.oasisopen.sca.ResponseDispatch;
import org.oasisopen.sca.annotation.*;


/**
 * Java component implementation
 * 1 service with interface Service1AsyncServer
 * 1 reference (1..1) with interface DataStore
 *
 * Async server implementation of the Service1WithFaults service which invokes ResponseDispatch.sendFault() method more than once
 *
 */
@Service(Service1WithFaultsAsyncServer.class)
public class Service1AsyncServerMultiFaultImpl implements Service1WithFaultsAsyncServer {
	
	private volatile ResponseDispatch<String> responseHandler = null;
	
	@Property(required=true)
	public volatile String serviceName = "service1";
	@Reference
	public volatile DataStore dataStore = null;
	
	/*
	public String operation1(String input) throws BusinessFault1, BusinessFault2 {
		return serviceName + " operation1 invoked";
	}
	*/

	public void operation1Async(String input, ResponseDispatch<String> handler) {
		// Store the ResponseDispatch object
		responseHandler = handler;
		
		// Now kick off processing on a separate thread that will dispatch the response some time after this
		// initial method returns...
		runResponseThread();
		
		// return
		return;
	} // end method operation1Async
	
	/**
	 * Method used to run a separate thread, to invoke the ResponseDispatch
	 */
	private void runResponseThread() {
		
		int invocationCount = 2;	// # of threads to use
		long maxWaitTime = 5000;	// Max wait time for completion = 5sec
		
		// Run the tests using a ThreadPoolExecutor
		ThreadPoolExecutor theExecutor = new ThreadPoolExecutor( 	invocationCount, invocationCount,
													                maxWaitTime, TimeUnit.MILLISECONDS,
													                new ArrayBlockingQueue<Runnable>( invocationCount ) );
		
		// Perform the invocations on separate thread...
		theExecutor.execute( new separateThreadInvoker() );

	} // end method runResponseThread

	/**
	 * An inner class which performs operations on the ResponseDispatch on a separate thread
	 */
	private class separateThreadInvoker implements Runnable {
		
		long pauseTime = 1000;		// Pause interval to allow initiating thread to complete
				
		public separateThreadInvoker() {
			super();
		} // end constructor

		public void run() {

			// Wait for a short time to ensure that the invoking thread has time to return
			try {
				Thread.sleep(pauseTime);
			} catch (InterruptedException e) {
				// Nothing to do here...
			} // end try
			
			// Invoke the reponseHandler object to return the response
			responseHandler.sendFault( new BusinessFault1("First fault thrown") );

			String secondResponse = "";
			// Second invocation of the reponseHandler.sendResponse() method for a second time
			try {
				responseHandler.sendFault( new BusinessFault2("Second fault thrown") );
				secondResponse += " unexpected - second invocation of sendFault did not throw exception";
			} catch ( IllegalStateException e ) {
				secondResponse += " expected IllegalStateException thrown on second invocation of sendFault";
			} catch ( Throwable t ) {
				secondResponse += " unexpected exception thrown on second invocation of sendFault: " +
				                  t.getClass().getName() + t.getMessage();
			} // end try
			
			// Store the final action sequence data...
			dataStore.storeData( serviceName, secondResponse );
			
		} // end method run
		
	} // end class separateThreadInvoker

}
