/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.ServiceReference;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 1 reference "reference1" with interface Service1 
 * 
 * This implementation performs invocations of the cast() method of the ComponentContext API and
 * checks that the results conform to a series of specification requirements
 */
@Service(Service1.class)
public class Service1CCcastImpl implements Service1 {
	
	private ComponentContext componentContext;
	private String responseAccumulator = "";
	
	@Property
	public String serviceName = "service1";
	@Reference
	public Service1 reference1;	

	public Service1CCcastImpl() {
		super();
	}
	
	/**
	 * Setter for the ComponentContext used by this implementation
	 * @param theContext - the ComponentContext
	 */
	@Context
	public void setContext( ComponentContext theContext ) {
		componentContext = theContext;
	} // end method setContext

	public String operation1(String input) {
		responseAccumulator += serviceName + " operation1 invoked cast";
		
		if( componentContext == null ) return responseAccumulator + " injected componentContext null";
		
		// JCA-TA-9035 - IllegalArgumentException thrown if supplied target is not an SCA reference proxy
		try {
		    componentContext.cast( componentContext );
		    responseAccumulator += " did not get IllegalArgumentException for invalid reference proxy";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for invalid reference proxy";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for invalid reference proxy: " + e.toString();
		} // end try	
		
		// JCA-TA-9034 - returns a valid ServiceReference typed by the same interface as the supplied proxy
		try {
		    ServiceReference<Service1> serviceReference = componentContext.cast(reference1);
		    if( serviceReference == null ) {
			    responseAccumulator += " unexpected null ServiceReference";
		    } else {
		    	responseAccumulator += " expected non-null ServiceReference";
		    	if ( serviceReference.getBusinessInterface().equals(Service1.class) ) {
		    		responseAccumulator += " ServiceReference has correct business interface";
		    	} else {
		    		responseAccumulator += " ServiceReference does not have correct business interface";
		    	} // end if
		    } // end if
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " unexpected IllegalArgumentException received for valid proxy";
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for valid proxy: " + e.toString();
		} // end try		

		return responseAccumulator;
	} // end operation1

} // end class
