/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 1 reference "reference1" (1..1) with interface Service1
 * 1 reference "dataStore" (1..1) with interface DataStore
 * 
 * This implementation acts as a controller for a lifecycle test on the component implementation
 * which is the target of reference1
 * - reference1 is invoked 
 * - when the reference1 invocation has completed, the dataStore is queried for the result data
 *   which has been stored in the data store by the lifecycle component implementation
 */
@Service(Service1.class)
public class lifecycleControllerImpl implements Service1 {
	
	@Property
	public String serviceName = "service1";
	@Property
	public String storeName	= null;
	// Required = true -> multiplicity 1..1
	@Reference(required=true)
	public Service1 reference1 	= null;
	@Reference(required=true)
	public DataStore dataStore 	= null;

	public String operation1(String input) {

		// Invoke the target lifecycle service
		// This MAY generate an exception for some of the tests - absorb this here since
		// the "real result" is placed into the data store and is retrieved from there
		// irrespective of the exception
		try {
			reference1.operation1( input );
		} catch (Exception e ) {
			System.out.println("lifecycle controller got exception from lifecycle service: " + e.getMessage());
			e.printStackTrace();
		} // end try
		
		// TODO Probably need to wait here for the lifecycle to complete...
		
		// Retrieve result data from the data store
		String result = dataStore.retrieveData( storeName );
		
		return result;
	}

} // end class lifecycleControllerImpl
